#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
# Based on dannipenguin's threaded queue code at http://dannipenguin.livejournal.com/288663.html
#
# Example of use:
"""
class MainApp(ThreadedClass):
    def __init__(self):
        self.printStuff('a', callback=self.donePrinting)
        self.printStuff('b', callback=self.donePrinting)
        self.printStuff('c', callback=self.donePrinting)
    
    @async_method
    def printStuff(self, stuff):
        for i in range(5):
            print stuff
    
    def donePrinting(self, return_code):
        print "Done"

gobject.threads_init()
app = MainApp()
gtk.main()
"""

from threading import Thread
from Queue import Queue
#from multiprocessing import Process
from vineyard import crashhandler
import gobject
import sys,traceback

def _print_exception_details():
    et, ev, tb = sys.exc_info()
    while tb:
        co = tb.tb_frame.f_code
        filename = str(co.co_filename)
        line_no =  str(traceback.tb_lineno(tb))
        print "%s:%s" % (filename, line_no)
        tb = tb.tb_next
    print "%s: %s" % (et, ev)

class ThreadedClass_queued(object):
    def __init__(self):
        self.q = Queue()

        t = Thread(target=self._thread_worker)
        t.setDaemon(True)
        t.start()

    def run_in_thread(self, func, *args, **kwargs):
        """Add a request to the queue. Pass callback= and/or error= as
           keyword arguments to receive return from functions or exceptions.
        """
        try:
            self.q
        except AttributeError:
            ThreadedClass.__init__(self)

        self.q.put((func, args, kwargs))

    def _thread_worker(self):
        while True:
            request = self.q.get()
            self.do_request(request)
            self.q.task_done()

    def do_request(self, (func, args, kwargs)):
        if 'callback' in kwargs:
            callback = kwargs['callback']
            del kwargs['callback']
        else:
            callback = None

        if 'error' in kwargs:
            error = kwargs['error']
            del kwargs['error']
        else:
            error = None

        try:
            r = func(*args, **kwargs)
            if not isinstance(r, tuple): r = (r,)
            if callback: self.do_callback(callback, *r)
        except Exception, e:
            if error: self.do_callback(error, e)
            else: print "Unhandled error:", e

    def do_callback(self, callback, *args):
        def _callback(callback, args):
            callback(*args)
            return False

        gobject.idle_add(_callback, callback, args)

class ThreadedClass_multi(object):
    def run_in_thread(self, function, *args, **kwargs):
        Thread(target=self._thread_run, args=(function,)+args, kwargs=kwargs).start()
    
    def _thread_run(self, function, *args, **kwargs):
        if 'callback' in kwargs:
            callback = kwargs['callback']
            del kwargs['callback']
        else:
            callback = None

        if 'error' in kwargs:
            error = kwargs['error']
            del kwargs['error']
        else:
            error = None

        try:
            r = function(*args, **kwargs)
            if not isinstance(r, tuple): r = (r,)
            if callback: self._thread_do_callback(callback, *r)
        except Exception as e:
            print "Error occured"
            if error:
                self._thread_do_callback(error, e)
            else:
                print "There was an error running the threaded function:", e
                print _print_exception_details()
                #crashhandler.show_error_window("There was an error running the threaded function, the error was:\n%s: %s" % (error, e))
    
    def _thread_do_callback(self, callback, *args):
        def _callback(callback, args):
            callback(*args)
            return False

        gobject.idle_add(_callback, callback, args)

def async_method(func):
    """ Makes the given function be run in a separate thread.
        Use with decorators.
    """

    def bound_func(obj, *args, **kwargs):
        obj.run_in_thread(func, obj, *args, **kwargs)

    return bound_func

def mainloop_method(func):
    def wrapped_func(*args, **kwargs):
        return gobject.idle_add(idle_func, *args, **kwargs)
    
    def idle_func(*args, **kwargs):
        func(*args, **kwargs)
        return False
    
    return idle_func

def execute_in_mainloop(*args, **kwargs):
    def _function():
        try:
            args[0](*args[1:], **kwargs)
        except TypeError:
            print "Couldn't run function in mainloop! Arguments were: "+\
                   "{0}({1}, {2}".format(args[0], args[1:], kwargs)
        return False
    gobject.idle_add(_function)

class ThreadedClass(ThreadedClass_multi):
    pass
