#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
import gobject, gtk, pango, sys, re, wine
from vineyard import common, async

RUNNING_PROGRAMS = {}
THREADING = async.ThreadedClass()

class MonitoredProgram:
    def __init__(self, command, name):
        process = wine.run(command)
        if process == False:
            self.pid = False
            self.name = name
            self._show_dialog_program_error_question(offer_info=False)
            return None
        RUNNING_PROGRAMS[process.pid] = process
        RUNNING_PROGRAMS[process.pid].name = self.name = name
        self.process_stderr = ''
        gobject.timeout_add(500, self._start_monitoring)
        self.pid = process.pid
    
    def _read_stderr(self):
        print "Reading stderr"
        process = RUNNING_PROGRAMS[self.pid]
        if not process.stderr.closed:
            stderr_buffer = process.stderr.read(1)
            while stderr_buffer != '':
                self.process_stderr += stderr_buffer
                stderr_buffer = process.stderr.read(1)
        return len(self.process_stderr.strip())
        
    def _start_monitoring(self):
        process = RUNNING_PROGRAMS[self.pid]
        
        process_done = False
        process.poll()
        if process.returncode != None:
            THREADING.run_in_thread(self._read_stderr, callback = self._determine_if_error)
            return False
        else:
            return True
    
    def _determine_if_error(self, return_value):
        print "Determining if error happened"
        if return_value:
            self._show_dialog_program_error_question()
    
    def _show_dialog_program_error_question(self, offer_info=True):
        dialog = gtk.MessageDialog(parent=common.get_main_window(),
                                   flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                                   type=gtk.MESSAGE_ERROR,
                                   buttons=gtk.BUTTONS_OK)
        dialog.set_icon_name('vineyard-preferences')
        dialog.set_markup('<span weight="bold" size="larger">%s</span>' % \
            _("The program exited with an error"))
        if self.name == 'winetricks':
            dialog.format_secondary_markup(unicode('The installation program did ' + \
                'not exit cleanly. You may have success with simply running it again.'))
        else:
            dialog.format_secondary_markup(unicode('The program <b>%(program)s</b> did ' + \
                'not exit cleanly and any work or progress you may have ' + \
                'done in the program may have been lost.') % \
                {'program': common.escape_xml(self.name)})
        settings = gtk.settings_get_default()
        settings.set_property('gtk-alternative-button-order', True)
        if offer_info:
            button_info = gtk.Button("More info")
            dialog.add_action_widget(button_info, 10)
            dialog.set_alternative_button_order([10, gtk.RESPONSE_OK])
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.connect("response", self._on_dialog_main_response)
        dialog.show_all()

    def _on_dialog_main_response(self, dialog, response):
        dialog.destroy()
        process = RUNNING_PROGRAMS[self.pid]
        errortext = self.process_stderr.split('\n')
        #errortext = process.stderr.read().split('\n')
        if response == 10:
            show_explanation = False
            # See if we can find the cause of the error
            missing_dlls = []
            for line in errortext:
                match = re.match(r'(?i)^err:module:import_dll .*? (\w+\.\w{3})', line)
                if match:
                    missing_dlls.append(match.groups()[0].lower())
            missing_dlls = list(set(missing_dlls))
            # We found some missing dlls, alter display to match and find detailed information
            if len(missing_dlls):
                missing_packages = []
                libraryPackages = wine.libraries.libraryPackages
                text = _("A possible cause of the error could be one or more of the following:")+"\n"
                for dll in missing_dlls:
                    text += "\n\t"+_("The library file (DLL) <b>%s</b> wasn't found.") % dll
                    package = [ k for (k, v) in libraryPackages.iteritems() if dll in v ]
                    if len(package):
                        missing_packages.append(package[0])
                # Remove duplicate package findings
                missing_packages = list(set(missing_packages))
                if len(missing_packages):
                    if len(missing_packages) == 1:
                        text += str("\n\n"+_("Based on this output it looks like " + \
                            "you need to install <b>%(package)s</b> in order " + \
                            "to get <b>%(program)s</b> to run successfully.")) % \
                            {'package': missing_packages[0], 'program': common.escape_xml(process.name)}
                    else:
                        list_of_packages = _(", ").join([ "<b>%s</b>" % i for i in missing_packages[:-1] ])+ \
                                           _(" and ")+missing_packages[-1]
                        text += str("\n\n"+_("Based on this output it looks like " + \
                            "you need to install %(packages)s in order to get " + \
                            "<b>%(program)s</b> to run successfully.")) % \
                            {'packages': list_of_packages, 'program': common.escape_xml(process.name)}
                show_explanation = True
            # We didn't find anything, just display the raw output
            else:
                text = _("When running <b>%s</b> Wine returned the following errors:") % common.escape_xml(process.name)

            self._show_dialog_program_error(process.name,
                text,
                '\n'.join([ i.strip() for i in errortext ]).replace('\r', ''),
                show_explanation)
        else:
            sys.stdout.write('\n'.join([ i.strip() for i in errortext ]).replace('\r', '')+'\n')

    def _show_dialog_program_error(self, program_name, text, error, show_explanation=False):
        self._error_dialog = gtk.Dialog(_("Error information for %s") % program_name,
            parent = common.get_main_window(),
            flags = gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
            buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK))
        
        hbox = gtk.HBox(spacing=6)
        
        icon = gtk.Image()
        icon.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG)
        icon.set_alignment(0.5, 0.0)
        
        hbox.pack_start(icon, expand=False, fill=True)
        
        vbox = gtk.VBox()
        
        label = gtk.Label()
        label.set_line_wrap(True)
        label.set_alignment(0.0, 0.5)
        #label.set_markup('<span weight="bold" size="larger">%s</span>' % text)
        label.set_markup('%s' % text)
        
        vbox.pack_start(label, expand=False, fill=True)
        
        scrolledwindow = gtk.ScrolledWindow()
        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwindow.set_size_request(
            common.widget_get_char_width(scrolledwindow)*80,
            common.widget_get_char_height(scrolledwindow)*6)
        
        textview = gtk.TextView()
        textview.get_buffer().set_text(error)
        textview.modify_font(pango.FontDescription('monospace'))
        textview.set_cursor_visible(False)
        scrolledwindow.add(textview)
        
        if not show_explanation:
            vbox.pack_start(scrolledwindow, expand=True, fill=True)
        else:
            expander = gtk.Expander(_("Detailed error information"))
            expander.add(scrolledwindow)
            vbox.pack_start(expander, expand=True, fill=True)
        
        hbox.pack_start(vbox, expand=True, fill=True)
        
        self._error_dialog.vbox.pack_start(hbox, expand=True, fill=True)
        
        self._error_dialog.connect('destroy', self._on_dialog_program_error_response)
        self._error_dialog.connect('response', self._on_dialog_program_error_response)
        
        self._error_dialog.show_all()

    def _on_dialog_program_error_response(self, *args):
        self._error_dialog.destroy()
