#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
import copy, os
import gobject, gtk, widget, wine
from vineyard import common, program_handler, async

THREADING = async.ThreadedClass()
_DOWNLOADING = False
_DOWNLOAD_DIALOG = None
_DOWNLOAD_PACKAGES = None

INSTALLERS_PROGRAMS = [
    ('7zip', '7zip', "7-Zip", _('A file archiver with a high compression ratio')),
    ('adobeair', 'adobeair', "Adobe Air", _('An application platform')),
    ('ie6', 'ie', 'Internet Explorer 6', _('Microsoft Internet Explorer 6')),
    ('ie7', 'ie', 'Internet Explorer 7', _('Microsoft Internet Explorer 7')),
    ('mpc', 'mpc', 'Media Player Classic', _('A free audio and video player for Windows')),
    ('shockwave', 'shockwave', 'Shockwave Player', _('Adobe Shockwave Player')),
    ('quicktime72', 'quicktime', 'Quicktime 7.2', _('Apple Quicktime')),
    ('steam', 'steam', 'Steam', _('Steam Client from Valve')),
    ('vlc', 'vlc', 'VLC', _('VLC media player')),
    ('wmp9', 'wmp', 'Windows Media Player 9', _('Microsoft Windows Media Player 9 *')),
    ('wmp10', 'wmp', 'Windows Media Player 10', _('Microsoft Windows Media Player 10 *'))
]

# FIXME: This should be automaticallu matched against what the currently
#        installed version of winetricks can actually do.
INSTALLERS_EXTRAS = [
    ('d3dx9', 'directx', 'Direct3D', _("Direct3D from DirectX 9")),
    ('flash', 'flash', 'Adobe Flash', _('Adobe Flash Player ActiveX and Firefox plugins')),
    ('divx', 'package-x-generic', 'DivX', _('DivX video codec')),
    ('xvid', 'package-x-generic', 'Xvid', _('Xvid video codec')),
    ('gdiplus', 'windows-library', 'GDI+', _('Microsoft Windows GDI+ library')),
    ('mfc40', 'windows-library', 'Microsoft Foundation Classes', _('MFC module from Microsoft Visual C++')),
    ('dotnet11', 'windows-library', '.NET 1.1', _('Microsoft .NET *')),
    ('dotnet20', 'windows-library', '.NET 2.0', _('Microsoft .NET 2.0 *')),
    ('dotnet20sp2', 'windows-library', '.NET 2.0 SP2', _('Microsoft .NET 2.0 Service Pack 2 *')),
    ('dotnet30', 'windows-library', '.NET 3.0', _('Microsoft .NET 3.0*')),
    ('vb6run', 'windows-library', 'MS Visual Basic 6', _('Microsoft Visual Basic 6 Service Pack 6 runtime')),
    ('vb5run', 'windows-library', 'MS Visual Basic 5', _('Microsoft Visual Basic 5 runtime')),
    ('vb4run', 'windows-library', 'MS Visual Basic 4', _('Microsoft Visual Basic 4 runtime')),
    ('vb3run', 'windows-library', 'MS Visual Basic 3', _('Microsoft Visual Basic 3 runtime')),
    ('vb2run', 'windows-library', 'MS Visual Basic 2', _('Microsoft Visual Basic 2 runtime')),
#    ('mono20', 'mono', 'Mono 2.0', _('Open source .NET implementation')),
#    ('mono22', 'mono', 'Mono 2.2', _('Open source .NET implementation')),
#    ('mono24', 'mono', 'Mono 2.4', _('Open source .NET implementation')),
    ('mono26', 'mono', 'Mono 2.6', _('Open source .NET implementation')),
    ('vcrun2010', 'windows-library', 'MS Visual C++ 2010', _('Microsoft Visual C++ 2010 runtime libraries')),
    ('vcrun2008', 'windows-library', 'MS Visual C++ 2008', _('Microsoft Visual C++ 2008 runtime libraries')),
    ('vcrun2005', 'windows-library', 'MS Visual C++ 2005', _('Microsoft Visual C++ 2005 runtime libraries')),
    ('vcrun2003', 'windows-library', 'MS Visual C++ 2003', _('Microsoft Visual C++ 2003 runtime libraries')),
    ('vcrun6', 'windows-library', 'MS Visual C++ 6', _('Microsoft Visual C++ 2003 libraries')),
    ('msxml3', 'windows-library', 'MS XML 3', _('Microsoft XML Version 3')),
    ('msxml4', 'windows-library', 'MS XML 4', _('Microsoft XML Version 4')),
    ('msxml6', 'windows-library', 'MS XML 6', _('Microsoft XML Version 6')),
    ('ogg', 'package-x-generic', 'Ogg Filters/Codecs', _('flac, theora, speex, vorbix, schroedinger')),
    ('physx', 'package-x-generic', 'NVIDIA PhysX', _('NVIDIA/AEGIAs physics library')),
    ('corefonts', 'gtk-select-font', 'Base Windows Fonts', _('Microsofts Arial, Courier and Times')),
    ('liberation', 'gtk-select-font', 'Liberation Fonts', _('Liberation fonts from Red Hat')),
    ('tahoma', 'gtk-select-font', 'Tahoma Font', _('Microsofts Tahoma'))
]

INSTALLERS = (INSTALLERS_PROGRAMS + INSTALLERS_EXTRAS)

def get_installer_from_package_name(package_name):
    for name, icon, title, description in INSTALLERS:
        if name == package_name:
            return {
                'name': name,
                'icon': icon,
                'title': title,
                'description': description
            }
    return None

class Widget(widget.VineyardWidget):
    def __init__(self):
        widget.VineyardWidget.__init__(self)
        self.title = _("Installers")
        self.widget_should_expand = True
        self.installers_programs = INSTALLERS_PROGRAMS
        self.installers_extras = INSTALLERS_EXTRAS
        self._downloading = False
        self._downloading_dialog = None
        self._build_interface()
    
    def _build_interface(self):
        self.vbox = gtk.VBox()
        
        self.installers = []
        installers = []
        for n, (title, section) in enumerate((
            (_('Programs'), self.installers_programs),
            (_('Extras'), self.installers_extras)
        )):
            if n == 0:
                installers.append([None, None, '<header first>%s</header>' % title])
            else:
                installers.append([None, None, '<header>%s</header>' % title])
            self.installers.append(('', '', ''))
            
            for i in section:
                try:
                    if len(i) > 1:
                        icon = common.pixbuf_new_from_string(i[1])
                        if icon == None:
                            icon = common.pixbuf_new_from_string('package-x-generic')
                        installers.append( (i[0], icon, '%s\n<small>%s</small>' % (i[2], i[3])) )
                    else:
                        installers.append( i[0] )
                    self.installers.append(i)
                except IndexError:
                    print i
        
        self.list = common.list_view_new_icon_and_text(headers=None, ignore_first_column=True, number_of_text_columns=1, text_column=2, use_markup=True, select_multiple=True, items=installers)
        self.vbox.pack_start(self.list)
        self.license_notice_label = gtk.Label(_('* Legally requires that you own a Windows license'))
        self.license_notice_label.set_alignment(1.0, 0.5)
        self.vbox.pack_start(self.license_notice_label, False, True)
        self.hbox_buttons = gtk.HBox()
        self.hbox_buttons.set_spacing(6)
        self.button_install = common.button_new_with_image('gtk-add', label=_("_Install"), use_underline=True)
        self.button_install.set_sensitive(False)
        self.hbox_buttons.pack_start(self.button_install, True, True)
        self.vbox.pack_start(self.hbox_buttons, False, True)
        
        self.list.connect('changed', self.list_changed)
        self.button_install.connect('clicked', self.button_install_clicked)
        
        self.frame = common.frame_wrap(self.vbox, self.title)
        self.pack_start(self.frame, True, True)
        
        self.sizable = []
        
        self.show_all()
        self.license_notice_label.hide()
    
    def _gui_set_program_selected(self, state):
        self.button_install.set_sensitive(state)
    
    def list_changed(self, listwidget, treeview, active_nr, active_text):
        selected = [ self.installers[i][3] for i in self.list.get_active() ]
        if any(( i.strip().endswith('*') for i in selected )):
            self.license_notice_label.show()
        else:
            self.license_notice_label.hide()
        self._gui_set_program_selected(active_nr != None)
                
    def button_install_clicked(self, button):
        packages = [ self.installers[i][0] for i in self.list.get_active() ]
        run_winetricks(packages, parent_window = common.get_main_window())

def run_winetricks(packages, parent_window=None):
    global _DOWNLOADING, _DOWNLOAD_DIALOG, _DOWNLOAD_PACKAGES
    if not wine.util.get_internet_available():
        return _show_no_internet_dialog()
    
    _DOWNLOAD_PACKAGES = packages
    
    if not wine.winetricks_installed():
        dialog = gtk.Dialog(_("Downloading winetricks"),
                            parent = common.get_main_window(),
                            flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
                            buttons = None)
        hbox = gtk.HBox()
        hbox.set_spacing(6)
        dialog.vbox.pack_start(hbox, False, True)
        
        _DOWNLOADING = True
        
        if hasattr(gtk, 'Spinner'):
            spinner = gtk.Spinner()
            hbox.pack_start(spinner, False, False)
            spinner.show()
            spinner.start()
        else:
            progressbar = gtk.ProgressBar()
            progressbar.set_pulse_step(0.1)
            dialog.vbox.pack_start(progressbar, False, True)
            progressbar.show()
            gobject.timeout_add(120, _downloading_update, progressbar)
        
        label = gtk.Label(_('Downloading winetricks...'))
        hbox.pack_start(label, False, True)
        label.show()
        hbox.show()
        dialog.show()
        _DOWNLOAD_DIALOG = dialog
        
        THREADING.run_in_thread(wine.update_winetricks, callback = _run_winetricks)
    else:
        _run_winetricks()

def _no_internet_available():
    # FIXME: Show a dialog
    return None

def _downloading_update(progressbar):
    global _DOWNLOADING
    progressbar.pulse()
    return _DOWNLOADING

def _run_winetricks(return_value = None):
    global _DOWNLOADING, _DOWNLOAD_DIALOG, _DOWNLOAD_PACKAGES
    _DOWNLOADING = True
    if _DOWNLOAD_DIALOG != None:
        _DOWNLOAD_DIALOG.destroy()
    
    first_package_title = get_installer_from_package_name(
        _DOWNLOAD_PACKAGES[0]
    )['title']
    
    if len(_DOWNLOAD_PACKAGES) > 1:
        title = _("Installing {package} and others").format(
            package = first_package_title
        )
    else:
        title = _("Installing {package}").format(
            package = first_package_title
        )
    dialog = gtk.MessageDialog(
        parent = common.get_main_window(),
        flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
        type = gtk.MESSAGE_INFO,
        buttons = gtk.BUTTONS_CANCEL)
    
    _DOWNLOADING = True
    
    dialog.image_widget = gtk.Image()
    dialog.image_widget.set_from_pixbuf(
        common.pixbuf_new_from_string(
            'package-x-generic',
            icon_size = gtk.ICON_SIZE_DIALOG,
        )
    )
    dialog.set_image(dialog.image_widget)
    
    dialog.set_markup('<b>{0}</b>'.format(title))
    dialog.format_secondary_markup(_(
        "The package files will be cached locally for installation."
    ))
    
    dialog.progressbox = gtk.VBox()
    dialog.vbox.pack_start(dialog.progressbox, False, True)
    
    dialog.progressbar = gtk.ProgressBar()
    dialog.progressbar.set_pulse_step(0.1)
    dialog.progressbox.pack_start(dialog.progressbar, False, True)
    
    #dialog.label_2 = gtk.Label(_(
    #    "The package files will be cached locally for installation."
    #))
    #dialog.label_2.set_alignment(0.0, 0.5)
    #dialog.progressbox.pack_start(dialog.label_2, False, False)
    
    dialog.button = dialog.action_area.get_children()[0]
    
    dialog.connect('response', _cancel_winetricks)
    
    #if len(_DOWNLOAD_PACKAGES) > 1:
    #    dialog.label = gtk.Label(
    #        _('Downloading installer {num} of {amount}...').format(
    #            num = 1,
    #            amount = len(_DOWNLOAD_PACKAGES)
    #        )
    #    ))
    #else:
    dialog.show_all()
    _DOWNLOAD_DIALOG = dialog
    
    _DOWNLOAD_DIALOG.process = _create_winetricks_process(_DOWNLOAD_PACKAGES)
    _DOWNLOAD_DIALOG.process.started_install = False
    gobject.timeout_add(120, _downloading_installer, dialog)

def _create_winetricks_process(packages):
    global _DOWNLOAD_DIALOG
    
    first_package_title = get_installer_from_package_name(packages[0])['title']
    
    if len(packages) > 1:
        package_name = _("{first_package} and others").format(
            first_package = first_package_title
        )
    else:
        package_name = first_package_title
    
    return wine.monitor.Winetricks(
        packages,
        name = _("Installer for {package}").format(
            package = package_name
        )
    )

def _winetricks():
    global _DOWNLOADING, _DOWNLOAD_DIALOG
    _DOWNLOAD_DIALOG.process.wait()

def _downloading_installer(dialog):
    global _DOWNLOADING, _DOWNLOAD_DIALOG
    
    status = dialog.process.get_output()
    if status is None:
        if _DOWNLOAD_DIALOG.process.started_install:
            dialog.format_secondary_markup(_(
                "Installing package..."
            ))
            dialog.progressbar.set_text(_("Installing..."))
        dialog.progressbar.pulse()
    else:
        _DOWNLOAD_DIALOG.process.started_install = True
        
        if status[0] == 'downloading':
            dialog.format_secondary_markup(_(
                "The package files will be cached locally for installation."
            ))
            dialog.progressbar.set_text(_("Downloading {file}").format(
                file = status[1]
            ))
            dialog.progressbar.set_fraction(
                float(status[2] / 100.)
            )
        
        elif status[0] == 'already installed':
            dialog.progressbar.set_text(_("{package} already installed, skipping.").format(
                package = get_installer_from_package_name(status[1])['title']
            ))
            dialog.progressbar.set_fraction(1.0)
        
        elif status[0] == 'extracting':
            dialog.progressbar.set_text(_("Unpacking installation files..."))
            dialog.progressbar.pulse()
        
        elif status[0] == 'faulty file':
            dialog.progressbar.set_text(_("Mismatching file already exists, trying again..."))
            os.remove(status[1])
            dialog.progressbar.set_fraction(0.0)
            _DOWNLOAD_DIALOG.process = _create_winetricks_process(_DOWNLOAD_PACKAGES)
            _DOWNLOAD_DIALOG.process.started_install = False
        
        elif status[0] == 'success':
            dialog.progressbar.set_text(_("{package} successfully installed.").format(
                package = get_installer_from_package_name(status[1])['title']
            ))
            dialog.progressbar.set_fraction(1.0)
        
        elif status[0] == 'done':
            dialog.format_secondary_markup(_(
                "Everything successfully installed."
            ))
            dialog.progressbar.set_text(_("Done."))
            dialog.progressbar.set_fraction(1.0)
            dialog.button.set_label('ok')
            dialog.button.set_use_stock(True)
        
    _DOWNLOADING = dialog.process.is_alive()
    
    return True

def _cancel_winetricks(dialog, response):
    global _DOWNLOADING, _DOWNLOAD_DIALOG
    if _DOWNLOADING:
        _DOWNLOAD_DIALOG.process.kill()
    _DOWNLOAD_DIALOG.destroy()
    