#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
import copy
import gtk, widget, wine
from vineyard import common

class Widget(widget.VineyardWidget):
    def __init__(self):
        widget.VineyardWidget.__init__(self)
        self.title = "Themes"
        self.widget_should_expand = True
        self.settings_key = 'themes'
        self._build_interface()
    
    def _build_interface(self):
        self.list = common.list_view_new_text(select_multiple=False)
        self.pack_start(self.list)
        self.hbox = gtk.HBox()
        self.hbox.set_spacing(6)
        self.pack_start(self.hbox, False, False)
        self.button_add = gtk.Button(stock=gtk.STOCK_ADD)
        self.button_add.set_sensitive(False)
        self.hbox.pack_start(self.button_add)
        self.button_remove = gtk.Button(stock=gtk.STOCK_REMOVE)
        self.button_remove.set_sensitive(False)
        self.hbox.pack_start(self.button_remove)
        
        self.list.connect('changed', self.list_changed)
        self.button_add.connect('clicked', self.button_add_clicked)
        self.button_remove.connect('clicked', self.button_remove_clicked)
        
        self.show_all()
    
    def list_changed(self, listwidget, treeview, active_nr, active_text):
        self.button_remove.set_sensitive(active_nr != None)
    
    def button_remove_clicked(self, button):
        libraries_to_remove = self.list.get_active_text()
        self.list.remove_by_text(libraries_to_remove)
        try:
            for library in libraries_to_remove:
                self.settings[self.settings_key]['new_list'].remove(library)
        except ValueError:
            print "Can't remove non-existing library override. How did this happen?"
        self.gobject.emit('settings-changed', self.settings_key, self.set_function, (self.settings[self.settings_key],))
    
    def button_add_clicked(self, button):
        new_library = self.comboboxentry.child.get_text()
        self.settings[self.settings_key]['new_list'].append(new_library)
        self.settings[self.settings_key]['new_list'].sort()
        self.list.set_from_list(self.settings[self.settings_key]['new_list'])
        self.button_add.set_sensitive(False)
        self.gobject.emit('settings-changed', self.settings_key, self.set_function, (self.settings[self.settings_key],))
    
    def set_function(self, settings):
        new_list, original_list = settings['new_list'], settings['original_list']
        # Remove
        for library in original_list:
            if library not in new_list:
                wine.libraries.set_override(library, False)
        # Add
        for library in new_list:
            if library not in original_list:
                wine.libraries.set_override(library, True)
    
    def load_settings(self):
        themes = wine.appearance.list_themes()
        self.settings[self.settings_key] = {'new_list': themes, 'original_list': copy.copy(themes)}
    
    def fill_widgets(self):
        self.list.clear()
        for theme in self.settings[self.settings_key]['original_list']:
            self.list.append_text(theme)
        
        self.gobject.emit('settings-loaded', self.settings_key, (self.settings[self.settings_key],))
